import javax.media.opengl.*;            //Fuer GL-Objekt und GLEventListener
import javax.media.opengl.glu.*;        //Fuer OpenGL Utility Library 
import com.sun.opengl.util.*;           //Fuer OpenGL Utility Toolkit   

import java.awt.*;                      //Fuer Klasse Point etc.
import java.awt.event.*;                //Fuer AWT-Listener

import framework.fps.*;                 //FpsCounter
import framework.controls.*;            //Keyboard-/MouseHandler
import framework.jogl.*;                //JOGLCamera, JOGLText
import framework.utils.*;               //3D-Ursprung, Fokus-Quadrat

/**
* Vordefinierter JOGLMainListener mit bereits folgenden 
* vorimplementierten Komponenten des Frameworks:
* 
* GLU, GLUT, KeyboardHandler, 2 mal den MouseHandler, 
* FpsCounter, JOGLText, JOGLCamera, 
* JOGLOrigin (3D-Ursprung), 
* JOGLFocus (Kamera-Orientierungshilfe).
*
* OpenGL ist bereits mit haeufig verwendeten
* Einstellungen vorinitialisiert!
*/
public class HWL_balls2D extends JOGLMainListener 
{
    // OpenGL Object
    private GL gl                           = null;
    
    // OpenGL Utility Library
    private GLU glu                         = new GLU();
      
    // OpenGL Utility Toolkit
    private GLUT glut                       = new GLUT();
    
    // Framework KeyboardHandler
    private KeyboardHandler keyboard        = new KeyboardHandler();
    
    // Framework MouseHandler fuer Camera (Relative Koordinaten -> Erweiterte Variante)
    private MouseHandler mouseCam           = new MouseHandler(true);
    
    // Framework MouseHandler fuer normalen Mauscursor (normale Variante)
    private MouseHandler mouseNorm          = new MouseHandler(false);
        
    // Framework FpsCounter zum Messen der Frames per second
    private FpsCounter   fpsCounter         = new FpsCounter();
    
    // Framework JOGLText zur Unterstuetzung von textausgabe
    private JOGLText   joglText             = new JOGLText();
            
    // Framework JOGLCamera
    private JOGLCamera camera               = null;
 
    //Framework 3D-Ursprung
    private JOGLOrigin origin               = new JOGLOrigin();
    
    //Framework Fokus-Quadrat zur Kamera-Orientierungshilfe
    private JOGLFocus camFocus              = new JOGLFocus(false);
    
    //Speichert relative Koordinaten des MouseHandlers
    private Point relKoord                  = null;
    
    //Speichert Status, ob Hinweismenue aktiv oder nicht
    private boolean showInfoMessage         = true;
    
    /**
    * Der HeavyweightListener erwartet die Angabe
    * einer Klasse, welche das Interface JOGLMainFrameInterf
    * implementiert. Im Normalfall sollte hier eine Instanz des
    * JOGLMainFrames angegeben werden.
    *
    * @param mainFrame  Interface JOGLMainFrameInterf 
    */
    public HWL_balls2D(JOGLMainFrameInterf mainFrame)
    {
        //Weitergabe des JOGLMainFrameInterf an die Oberklasse
        //JOGLMainListener
        super(mainFrame);
    }
    
    //--------------------------------------------------------------
    // **** Eigene globale Methoden/Variablen hier ****
    //--------------------------------------------------------------
    
    balls my_balls = new balls();
    //balls_with_texture my_balls = new balls_with_texture();
    
    //--------------------------------------------------------------
    // **** Ende eigene globale Methoden/Variablen ****
    //--------------------------------------------------------------
    
    /** GLEventListener-Methode (Siehe JOGL API)*/
    public void init(GLAutoDrawable glDrawable)
    {
        // AWT Keylistener, MouseListener und MouseMotionListener
        // registrieren. 
        glDrawable.addKeyListener(this);
        glDrawable.addMouseListener(this);
        glDrawable.addMouseMotionListener(this);
        
        /* Anfang OpenGL Standard-Initialisierungen */
      
        // OpenGL-Objekt holen
        gl  = glDrawable.getGL();
        
        gl.glShadeModel(GL.GL_SMOOTH);              
        gl.glClearColor(0.0f, 0.0f, 0.0f, 0.5f);    
        gl.glClearDepth(1.0f);                      
        gl.glEnable(GL.GL_DEPTH_TEST);              
        gl.glDepthFunc(GL.GL_LEQUAL);               
        gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
        gl.setSwapInterval(1);
        
        
        /* Ende OpenGL Standard-Initialisierungen */
        
        //--------------------------------------------------------------
        // **** Anfang eigene Initialisiserungen hier   ****
        //--------------------------------------------------------------
    
        my_balls.init_balls(gl,glu);
    
        //--------------------------------------------------------------
        // **** Ende eigene Initialisierungen Variablen ****
        //--------------------------------------------------------------
    }

    /** GLEventListener-Methode (Siehe JOGL API)*/
    public void display(GLAutoDrawable glDrawable)
    {
        // Fps messen
        fpsCounter.checkFps();
        
        // Wenn, Keyboardtaste "Escape" gedrueckt
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_ESCAPE)) 
        {
            //Programm herunterfahren
            mainFrame.shutdown();   
            
            //Loslassen der Escape-Taste simulieren.
            keyboard.setManualRelease(KeyEvent.VK_ESCAPE);
        }    
        
        //w,a,s,d,e,q Ansteuerung der Kamera 
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_W)) camera.cameraMoveForward();
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_S)) camera.cameraMoveBackward();
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_A)) camera.cameraMoveSideLeft();
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_D)) camera.cameraMoveSideRight();
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_Q)) camera.cameraHoverDown();
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_E)) camera.cameraHoverUp();
        
        //Relative Mousekoordinaten ermitteln
        relKoord = mouseCam.getRelativeMouseXY();
        
        //Wenn die Maus ueberhaupt bewegt wird
        if (relKoord.x != 0 || relKoord.y != 0)
        {
            //Camera Vertikale Bewegung berechnen
            camera.cameraLookV(-relKoord.y);
            //Camera Horizontale Bewegung berechnen
            camera.cameraLookH(relKoord.x);
        }
        
        /* Anfang OpenGL Standard-Initialisierungen */
        
        // OpenGL-Objekt holen
        gl = glDrawable.getGL();
        
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glLoadIdentity();
        
        /* Ende OpenGL Standard-Initialisierungen */
        
        //Orientierungshilfe zeichnen
        camFocus.drawFocus(gl);
        //Camera aktualisieren
        camera.cameraUpdate(glu);        
        //Matrix-Stack sichern
        gl.glPushMatrix();
        
        //Ursprung zeichnen (Diese Zeile kann bei Bedarf entfernt werden!)
        origin.drawOrigin(gl);
        
        //--------------------------------------------------------------
        // **** Anfang eigener Code fuer Hauptschleife hier    ****
        //
        // (Anstatt glLoadIdentity() sollte hier gl.popMatrix() benutzt
        //  werden um Konflikte mit der Camera zu vermeiden!)
        //--------------------------------------------------------------
    
        my_balls.display(gl, glu, glut);
        my_balls.game_physics(gl, glu, glut);

        //--------------------------------------------------------------
        // **** Ende eigener Code fuer Hauptschleife           ****
        //--------------------------------------------------------------
    
        
        // Wenn, Keyboardtaste "F1" gedrueckt
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_F1))
        {
            //Staus veraendern (Hinweismenue an/aus)
            showInfoMessage = !showInfoMessage;        
           
            //Loslassen der F1-Taste simulieren.
            keyboard.setManualRelease(KeyEvent.VK_F1);
        }
        
        //Steuerungshinweise anzeigen
        if (showInfoMessage)    
        {
            //Matrix zuruecksetzen, damit sich Text nicht mit Kamera bewegt
            gl.glLoadIdentity();
            
            //Textausgabe 
            joglText.drawText("Fps: ", 0.0f, 1.0f, 1.0f, -1.1f, 0.70f, -2.0f, gl, glut);
            joglText.drawText(""+fpsCounter.getActFps(), 1.0f, 0.0f, 0.0f, -0.75f, 0.70f, -2.0f, gl, glut);
            joglText.drawText("[Control]", 0.0f, 1.0f, 1.0f, -1.1f, 0.6f, -2.0f, gl, glut);
            joglText.drawText("Exit program:", 0.0f, 1.0f, 1.0f, -1.1f, 0.55f, -2.0f, gl, glut);
            joglText.drawText("Esc", 1.0f, 0.0f, 0.0f, -0.75f, 0.55f, -2.0f, gl, glut);
            joglText.drawText("Menu on/off:", 0.0f, 1.0f, 1.0f, -1.1f, 0.5f, -2.0f, gl, glut);
            joglText.drawText("F1", 1.0f, 0.0f, 0.0f, -0.75f, 0.5f, -2.0f, gl, glut);
            joglText.drawText("Moving Camera:", 0.0f, 1.0f, 1.0f, -1.1f, 0.45f, -2.0f, gl, glut);
            joglText.drawText("w, a, s, d, e, q", 1.0f, 0.0f, 0.0f, -0.75f, 0.45f, -2.0f, gl, glut);
            joglText.drawText("Change View:", 0.0f, 1.0f, 1.0f, -1.1f, 0.4f, -2.0f, gl, glut);
            joglText.drawText("Drag Right Mouse", 1.0f, 0.0f, 0.0f, -0.75f, 0.4f, -2.0f, gl, glut);
        }
    }
    
    /** GLEventListener-Methode (Siehe JOGL API)*/
    public void reshape(GLAutoDrawable glDrawable, int x, int y, int width, int height)
    {
        /* Anfang OpenGL Standard-Initialisierungen */
        
        // OpenGL-Objekt holen
        gl = glDrawable.getGL();
        
        if (height <= 0) height = 1;
        
        float ratio = (float)width / (float)height;

        gl.glViewport(0, 0, width, height);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        glu.gluPerspective(45.0f, ratio, 0.1, 1000.0);
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
    
        /* Ende OpenGL Standard-Initialisierungen */
        
        //Dem Mousehandler aktuelle Position des GLCanvas mitteilen
        mouseCam.canvasSizeUpdate(width, height);
        
        //Dem Mousehandler aktuelle Dimensionen des GLCanvas mitteilen
        mouseCam.canvasLocationUpdate(mainFrame.getCanvasLocationOnScreen().x,
                                      mainFrame.getCanvasLocationOnScreen().y);
        
        //Mauscursor zentrieren
        mouseCam.centerMouse();
        
        //Camera Starteinstellungen einstellen
        if (camera == null)
        {
            //Camera aufstellen an bestimmter Position und Blickrichtung
            camera = new JOGLCamera(glu, 1.0f, 1.0f, 5.0f, 0.0f, 0.0f);
            //Speed --> Bewegen der Kamera
            camera.setMoveSpeed(0.075f);
            //Speed --> Schweben der Kamera
            camera.setHoverSpeed(0.075f);
        }        
        //--------------------------------------------------------------
        // **** Anfang Eigener Code fuer Aktionen bei Groessenaenderung ****
        //--------------------------------------------------------------
    
        //                   - Code -
    
        //--------------------------------------------------------------
        // **** Ende eigener Code fuer Aktionen bei Groessenaenderung   ****
        //--------------------------------------------------------------
    }
     
    /** AWT-MouseListener (Siehe Java API)*/
    public void mouseEntered(MouseEvent e)
    {
        //Mousehandler aktuelle Position des GLCanvas mitteilen,
        mouseCam.canvasLocationUpdate(mainFrame.getCanvasLocationOnScreen().x,
                                       mainFrame.getCanvasLocationOnScreen().y); 
    }

    /** AWT-MouseListener (Siehe Java API)*/
    public void mouseReleased(MouseEvent e)
    {
        //Beide Mousehandler aktualisieren
        mouseCam.mouseReleasedUpdate(e);
        mouseNorm.mouseReleasedUpdate(e);
        
        //Wenn rechte Maus losgelassen
        if (!mouseCam.checkIsRightButtonDown())
        {
            //Orientierungshilfe ausschalten
            camFocus.setVisible(false);
            //Mauscursor wieder anzeigen
            mainFrame.showMouseCursor();
        }
    }
    
    /** AWT-MouseListener (Siehe Java API)*/
    public void mousePressed(MouseEvent e)
    {
        //Beide Mousehandler aktualisieren
        mouseCam.mousePressedUpdate(e);
        mouseNorm.mousePressedUpdate(e);
        
        //Wenn rechte Maus gedrueckt dann Mauscursor unsichtbar
        if (mouseCam.checkIsRightButtonDown())
            mainFrame.hideMouseCursor();
    }
    
    /** AWT-MouseMotionListener (Siehe Java API)*/
    public void mouseDragged(MouseEvent e)
    {
        //Mousehandler aktualisieren
        mouseNorm.mouseMovedUpdate(e); 
        
        //Wenn rechte Maus gedrueckt
        if (mouseCam.checkIsRightButtonDown())
        {
            //Orientierungshilfe einschalten
            camFocus.setVisible(true);
            //NUR HIER diesen Mousehandler aktualisieren 
            mouseCam.mouseMovedUpdate(e);
        }    
    }
    
    /** AWT-MouseMotionListener (Siehe Java API)*/
    public void mouseMoved(MouseEvent e)
    {
         //Mousehandler aktualisieren
         mouseNorm.mouseMovedUpdate(e);           
    }
    
    /** AWT-KeyListener (Siehe Java API)*/
    public void keyReleased(KeyEvent e)
    {
        // KeyboardHandler aktualisieren
        keyboard.keyReleasedUpdate(e);
    }
    
    /** AWT-KeyListener (Siehe Java API)*/
    public void keyPressed(KeyEvent e)
    {
        // KeyboardHandler aktualisieren
        keyboard.keyPressedUpdate(e);
    }
}