import javax.media.opengl.*;            //Fuer GL-Objekt und GLEventListener
import javax.media.opengl.glu.*;        //Fuer OpenGL Utility Library 
import com.sun.opengl.util.*;           //Fuer OpenGL Utility Toolkit   

import java.awt.*;                      //Fuer Klasse Point etc.
import java.awt.event.*;                //Fuer AWT-Listener

import framework.fps.*;                 //FpsCounter
import framework.controls.*;            //Keyboard-/MouseHandler
import framework.jogl.*;                //JOGLCamera, JOGLText ...

/**
* Vordefinierter JOGLMainListener mit bereits folgenden 
* vorimplementierten Komponenten des Frameworks:
* 
* GLU, GLUT, KeyboardHandler.
*
* OpenGL ist bereits mit haeufig verwendeten
* Einstellungen vorinitialisiert!
*/
public class LWL_balls2D extends JOGLMainListener 
{
    // OpenGL Objekt
    private GL gl                           = null;
    
    // OpenGL Utility Library
    private GLU glu                         = new GLU();
      
    // OpenGL Utility Toolkit
    private GLUT glut                       = new GLUT();
     
    // Framework KeyboardHandler
    private KeyboardHandler keyboard        = new KeyboardHandler();
    
    /**
    * Der LeightweightListener erwartet die Angabe
    * einer Klasse, welche das Interface JOGLMainFrameInterf
    * implementiert. Im Normalfall sollte hier eine Instanz des
    * JOGLMainFrames angegeben werden.
    *
    * @param mainFrame  Interface JOGLMainFrameInterf 
    */
    public LWL_balls2D(JOGLMainFrameInterf mainFrame)
    {
        //Weitergabe des JOGLMainFrameInterf an die Oberklasse
        //JOGLMainListener
        super(mainFrame);
    }
    
    //--------------------------------------------------------------
    // **** Eigene globale Methoden/Variablen hier ****
    //--------------------------------------------------------------
    
    balls my_balls = new balls();
    //balls_with_texture my_balls = new balls_with_texture();
    //balls_solution my_balls = new balls_solution();

    //--------------------------------------------------------------
    // **** Ende eigene globale Methoden/Variablen ****
    //--------------------------------------------------------------
    
    /** GLEventListener-Methode (Siehe JOGL API)*/
    public void init(GLAutoDrawable glDrawable)
    {
        // AWT Keylistener registrieren. Wichtig: Um den MouseHandler
        // benutzen zu koennen, muessen MouseListener und MouseMotionlister
        // ebenfalls registriert werden. Siehe HeavyweightListener!
         glDrawable.addKeyListener(this);
         
        /* Anfang OpenGL Standard-Initialisierungen */
      
        // OpenGL-Objekt holen
        gl  = glDrawable.getGL();
        
        gl.glShadeModel(GL.GL_SMOOTH);              
        gl.glClearColor(0.0f, 0.0f, 0.0f, 0.5f);    
        gl.glClearDepth(1.0f);                      
        gl.glEnable(GL.GL_DEPTH_TEST);              
        gl.glDepthFunc(GL.GL_LEQUAL);               
        gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
        gl.setSwapInterval(1);
        
        
        /* Ende OpenGL Standard-Initialisierungen */
        
        //--------------------------------------------------------------
        // **** Anfang eigene Initialisiserungen hier   ****
        //--------------------------------------------------------------
       
        my_balls.init_balls(gl,glu);
    
        //--------------------------------------------------------------
        // **** Ende eigene Initialisierungen Variablen ****
        //--------------------------------------------------------------
    }

    /** GLEventListener-Methode (Siehe JOGL API)*/
    public void display(GLAutoDrawable glDrawable)
    {
        // Wenn, Keyboardtaste "Escape" gedrueckt
        if (keyboard.checkIsKeyPressed(KeyEvent.VK_ESCAPE)) 
        {
            //Programm herunterfahren
            mainFrame.shutdown();   
            
            //Loslassen der Escape-Taste simulieren.
            keyboard.setManualRelease(KeyEvent.VK_ESCAPE);
        }    
        
        /* Anfang OpenGL Standard-Initialisierungen */
        
        // OpenGL-Objekt holen
        gl = glDrawable.getGL();
        
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glLoadIdentity();
        
        /* Ende OpenGL Standard-Initialisierungen */
        
        //--------------------------------------------------------------
        // **** Anfang eigener Code fuer Hauptschleife hier    ****
        //--------------------------------------------------------------
    
        my_balls.display(gl, glu, glut);
        my_balls.game_physics(gl, glu, glut);

        //--------------------------------------------------------------
        // **** Ende eigener Code fuer Hauptschleife           ****
        //--------------------------------------------------------------
    }
    
    /** GLEventListener-Methode (Siehe JOGL API)*/
    public void reshape(GLAutoDrawable glDrawable, int x, int y, int width, int height)
    {
        /* Anfang OpenGL Standard-Initialisierungen */
        
        // OpenGL-Objekt holen
        gl = glDrawable.getGL();
        
        if (height <= 0) height = 1;
        
        float ratio = (float)width / (float)height;

        gl.glViewport(0, 0, width, height);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        glu.gluPerspective(45.0f, ratio, 0.1, 1000.0);
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
    
        /* Ende OpenGL Standard-Initialisierungen */
        
        //--------------------------------------------------------------
        // **** Anfang Eigener Code fuer Aktionen bei Groessenaenderung ****
        //--------------------------------------------------------------
    
        //                   - Code -
    
        //--------------------------------------------------------------
        // **** Ende eigener Code fuer Aktionen bei Groessenaenderung   ****
        //--------------------------------------------------------------
    }
     
    /** AWT-KeyListener (Siehe Java API)*/
    public void keyReleased(KeyEvent e)
    {
        // KeyboardHandler aktualisieren
        keyboard.keyReleasedUpdate(e);
    }
    
    /** AWT-KeyListener (Siehe Java API)*/
    public void keyPressed(KeyEvent e)
    {
        // KeyboardHandler aktualisieren
        keyboard.keyPressedUpdate(e);
    }
}