/**
 * Improved version of balls-project
 * 	(with textures)
 */
import javax.media.opengl.*;            // for GL object and GLEventListener (GL)
import javax.media.opengl.glu.*;        // for OpenGL Utility Library (GLU)

import com.sun.opengl.util.*;           // for OpenGL Utility Toolkit (GLUT)   

import texture.TextureReader;
import java.io.IOException;
import java.util.Random;
import java.nio.*;						// FloatBuffer, IntBuffer


public class balls_with_texture {
	
	int totalball=6;
	double incr = 0.02f;		// time increment
	double g_acc = -9.8f;		// gravity constant
	Random ran = new Random();
	static int counter;
	
	int random(int n) {
		return ran.nextInt(n);
	}
	/* Define ball class for animation values */
	class BallClass {
		double times;			// time since start
		double xpos,ypos;		// actual position 
		double vx,vy;			// actual velocity 
		double ax,ay;			// acceleration, currently constant = (0,-9.8)
		double rotate,radius;
		BallClass () {
	   		rotate=5.0f;
			radius=0.4f;
			// we initialize with increasingly more negative 'times' 
			// >> the balls pop up one after another:
			times=-4*incr*counter;
			vy=0.0;             ypos=2.0;      ay=g_acc;
			vx=(random(20)+1);  xpos=-2.0;     ax=0.0;
			++counter;
			//System.out.println("xpos,ypos: " + xpos + ", " + ypos);
		}

	} ;
	
	BallClass[] ball = new BallClass[100];
	int ballnum=0;

	double zoom = -7;
	double leftwall_x = -5.6;
	double rightwall_x = +5.6;
	double floor_y = -4.25;
	double ceiling_y = 3.75;
    private int[] textures = new int[3];         // Storage For 3 Textures


	void init_balls(GL gl, GLU glu) {
        gl.glColorMaterial ( GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE );

        gl.glEnable      ( GL.GL_TEXTURE_2D );
        gl.glPixelStorei ( GL.GL_UNPACK_ALIGNMENT, 1 );

        loadGLTextures(gl, glu);

        //gl.glEnable ( GL.GL_CULL_FACE );
        gl.glClearColor ( 0.0f, 0.0f, 1.0f, 0.0f );			// blue background
        gl.glTranslated ( 0.0, 0.0, zoom );
        for (ballnum=0;ballnum < totalball; ballnum++) {
        	ball[ballnum] = new BallClass();
        }
             
/*        
        float LightDiffuse[] = { 0.8f, 0.25f, 0.0f, 1.0f };
        float LightAmbient[] = { 1.0f, 0.25f, 0.0f, 5.0f };
        float LightPosition[]= { 10.0f, 10.0f, -0.0f, 10.0f };
*/    
        float LightDiffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
        float LightAmbient[] = { 1.0f, 1.0f, 1.0f, 1.0f };
        float LightPosition[]= { 10.0f, 10.0f, -0.0f, 10.0f };
    
        gl.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, FloatBuffer.wrap(LightAmbient));
        gl.glLightfv(GL.GL_LIGHT2, GL.GL_DIFFUSE, FloatBuffer.wrap(LightDiffuse));
        gl.glLightfv(GL.GL_LIGHT2, GL.GL_POSITION,FloatBuffer.wrap(LightPosition));
        gl.glEnable(GL.GL_LIGHT1);
        gl.glEnable(GL.GL_LIGHT2);
        gl.glEnable(GL.GL_LIGHTING);

	
	}
	
	void display(GL gl, GLU glu, GLUT glut) 
	{
		float[] c_red = {0.5f,0.0f,0.0f};
		float[] c_white = {1.0f,1.0f,1.0f};
		vertical_line_at(leftwall_x, c_red, gl);
		vertical_line_at(rightwall_x, c_red, gl);
		horizontal_line_at(ceiling_y, c_red, gl);
		horizontal_line_at(floor_y, c_red, gl);
		
		// ---- Nachfolgendes Code-Snippet aktivieren fr Uebung U5 ------
		/*
		vertical_line_at(0.0, c_white, gl);		
		*/
		// ----------------------------------------------------------------


		gl.glBindTexture(GL.GL_TEXTURE_2D, textures[1]);	// works
//		gl.glBindTexture ( GL.GL_TEXTURE_2D, 0 );			// doesn't work, no texture
		gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,FloatBuffer.wrap(c_white));
		
		for (ballnum=0;ballnum < totalball; ballnum++) {
			if (ball[ballnum].times>=0) {
				
				gl.glPushMatrix  ( );
		        gl.glColor3f(1.0f, 1.0f, 1.0f);
				GLUquadric	q = glu.gluNewQuadric();
		  		glu.gluQuadricDrawStyle ( q, GLU.GLU_FILL   );
		  		glu.gluQuadricNormals   ( q, GLU.GLU_SMOOTH );
		  		glu.gluQuadricTexture   ( q, true);
				gl.glTranslated ( ball[ballnum].xpos, ball[ballnum].ypos, -11.0 );
				gl.glRotated(ball[ballnum].rotate,1,1,1);
				glu.gluSphere ( q, ball[ballnum].radius, 20, 20 );
				glu.gluDeleteQuadric ( q );
				gl.glPopMatrix ( );
				
			}
		}
		            
		//System.out.println(ball[0].times);

		texture_rectangle(gl);
        
		gl.glEnable    ( GL.GL_BLEND );
		
		// --- DoubleBuffering from CPP-template is not necessary ---
		//glutSwapBuffers ( );
		//gl.glClear   ( GL.GL_COLOR_BUFFER_BIT );
			
	} // display
	
	void game_physics(GL gl, GLU glu, GLUT glut)
	{
		int bn;
		for (bn=0;bn < totalball; bn++) {
		   ball[bn].rotate+=(ran.nextFloat()%10)*incr/0.02;
		   ball[bn].times+=incr;

		   // Game Physics:
		   ball[bn].xpos += ball[bn].vx * incr;
		   ball[bn].ypos += ball[bn].vy * incr; 
		   ball[bn].vx += ball[bn].ax * incr;
		   ball[bn].vy += ball[bn].ay * incr;
		   ball[bn].ax = 0.0f;
		   ball[bn].ay = g_acc;


		   // Reflections at floor, ceiling, left and right wall:
		   if ((ball[bn].ypos < floor_y+ball[bn].radius)) {			
	   		  ball[bn].vy = random(14);
		   }
		   else if ((ball[bn].ypos > ceiling_y-ball[bn].radius))  {		
			  ball[bn].vy = -random(5);
		   }

		   if ((ball[bn].xpos > rightwall_x-0.05-ball[bn].radius))  {		
			  ball[bn].vx = -random(5);
			  ball[bn].vy = random(9);
		   }
		   else if ((ball[bn].xpos < leftwall_x+0.05+ball[bn].radius))  {		
			  ball[bn].vx = random(5);
			  ball[bn].vy = random(9);
		   }


		}	// for

		//--- glutPostRedisplay() from CPP-template is not necessary ---
		//glut.glutPostRedisplay();

	}

	private void vertical_line_at(double x, float[] color, GL gl)
	{
	   	//gl.glColor3f(color[0],color[1],color[2]);
		gl.glBegin(GL.GL_LINES);
			gl.glMaterialfv(GL.GL_FRONT_AND_BACK,GL.GL_DIFFUSE,FloatBuffer.wrap(color));
			gl.glVertex3d(x,floor_y,-11.0);
	   	   	gl.glVertex3d(x,ceiling_y,-11.0);
		gl.glEnd();
	}	

	private void horizontal_line_at(double y, float[] color, GL gl)
	{
	   	//gl.glColor3f(color[0],color[1],color[2]);
		gl.glBegin(GL.GL_LINES);
			gl.glMaterialfv(GL.GL_FRONT_AND_BACK,GL.GL_DIFFUSE,FloatBuffer.wrap(color));
			gl.glVertex3d(leftwall_x,y,-11.0);
	   	   	gl.glVertex3d(rightwall_x,y,-11.0);
		gl.glEnd();
	}	

	// Load image and convert to textures
    private void loadGLTextures(GL gl, GLU glu) { 
        TextureReader.Texture texture;
        try {
            texture = TextureReader.readTexture("Data/rainbow-square.png");
        	//texture = TextureReader.readTexture("Data/fire.png");
        	//texture = TextureReader.readTexture("Data/marble.bmp");
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        gl.glGenTextures(3, textures, 0);                  // Create Three Textures
        // Create Nearest Filtered Texture
        gl.glBindTexture(GL.GL_TEXTURE_2D, textures[0]);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);
        gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, 3, texture.getWidth(), texture.getHeight(), 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE, texture.getPixels());

        // Create Linear Filtered Texture
        gl.glBindTexture(GL.GL_TEXTURE_2D, textures[1]);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
        gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, 3, texture.getWidth(), texture.getHeight(), 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE, texture.getPixels());

        // Create MipMapped Texture
        gl.glBindTexture(GL.GL_TEXTURE_2D, textures[2]);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR_MIPMAP_NEAREST);
        glu.gluBuild2DMipmaps(GL.GL_TEXTURE_2D, 3, texture.getWidth(), texture.getHeight(), GL.GL_RGB, GL.GL_UNSIGNED_BYTE, texture.getPixels());
    }

	void texture_rectangle(GL gl) 
	{
		// just to see whether we can map a texture to a rectangle behind
		// the balls:
	    gl.glBindTexture(GL.GL_TEXTURE_2D, textures[1]);	// works
	//	gl.glBindTexture ( GL.GL_TEXTURE_2D, 0 );			// doesn't work, no texture
	    gl.glBegin(GL.GL_QUADS);
	    float grey[]={0.5f, 0.5f, 0.5f,1.0f};
	    gl.glMaterialfv(GL.GL_FRONT_AND_BACK,GL.GL_DIFFUSE,FloatBuffer.wrap(grey));
	    // we want that the texture replaces the material actually used:
	    gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE);
	    gl.glNormal3f(0.0f, 1.0f, 1.0f);
	    gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(  5f, 3f,  -14f);     
	    gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(  5f,-3f,  -14f);         
	    gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f( -5f,-3f,  -14f);      
	    gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f( -5f, 3f,  -14f);
	    gl.glEnd();
	}
}
